from google.protobuf.json_format import MessageToJson
from google.api_core.exceptions import NotFound
import google.oauth2.service_account as service_account
from google.cloud import speech_v1p1beta1 as speech

import json
import os

import ftvstt.transcripts as transcripts
import ftvstt.exceptions as exceptions
import ftvstt.transcribers as transcribers


class Google(transcribers.Transcriber):
    def __init__(self):
        super().__init__()
        self.vocList = None
        self.authenticated = False

    rawType = "json"

    def authenticate(self, email, private_key):
        self.authenticated = False
        service_account_info = {
            "client_email": email,
            "type": "service_account",
            "token_uri": "https://oauth2.googleapis.com/token",
            "private_key": private_key}
        try:
            credentials = (service_account
                           .Credentials
                           .from_service_account_info(service_account_info))
            self.__client = speech.SpeechClient(credentials=credentials)
            (self.__client
                 .long_running_recognize(speech.types.RecognitionConfig(),
                                         speech.types.RecognitionAudio()))
        except NotFound:
            self.authenticated = True
        except Exception:
            pass
        return self.authenticated

    def authenticate_with_file(self, credentialsFilePath):
        self.credentialsFilePath = credentialsFilePath

        os.environ['GOOGLE_APPLICATION_CREDENTIALS'] = self.credentialsFilePath
        self.authenticated = False
        self.__client = speech.SpeechClient()
        self.authenticated = True

    def deauthenticate(self):
        if 'GOOGLE_APPLICATION_CREDENTIALS' in os.environ:
            os.environ.pop('GOOGLE_APPLICATION_CREDENTIALS')
        self.__client = None
        self.credentialsFilePath = None
        self.authenticated = False

    @exceptions.transcribe_error_handler
    def transcribe(
        self,
        inputPath,
        sample_rate_hertz=16000,
        encoding=speech.enums.RecognitionConfig.AudioEncoding.LINEAR16,
        enable_word_time_offsets=True,
        enable_speaker_diarization=True,
        lang="fr-FR"
    ):
        lang = self._make_language_compatible(lang)
        transcript = transcripts.Transcript()
        transcript.inputPath = inputPath
        transcript.provider = self.__class__

        if not self.authenticated:
            transcript.exception = exceptions.GoogleAuthError()
            transcript.success = False
            return transcript

        with open(transcript.inputPath, 'rb') as inputFile:
            content = inputFile.read()

        audio = speech.types.RecognitionAudio(content=content)

        config = speech.types.RecognitionConfig(
            encoding=encoding,
            sample_rate_hertz=sample_rate_hertz,
            language_code=lang,
            enable_word_time_offsets=enable_word_time_offsets,
            enable_speaker_diarization=enable_speaker_diarization
            )

        if self.vocList is not None:
            config.speech_contexts.add(phrases=self.vocList)

        transcript.success = True
        try:
            response = self.__client.long_running_recognize(config, audio)
            resultJson = MessageToJson(response.result())
            ascii_json = bytes(resultJson, 'ascii')
            transcript.raw = ascii_json.decode('unicode-escape')

            self.__class__.set_text(transcript)
            self.__class__.set_words(transcript)
        except Exception as e:
            transcript.success = False
            message = "An error has occured during "
            message += "Google Cloud transciption :\n"
            message += str(e)
            transcript.exception = exceptions.GoogleError(message)

        return transcript

    def set_text(transcript):
        data = json.loads(transcript.raw)
        transcript.text = ""
        for result in data['results']:
            if 'transcript' in result['alternatives'][0].keys():
                transcript.text += result['alternatives'][0]['transcript']
        return transcript.text

    def set_words(transcript):
        transcript.words = []
        transcript.speakers = []
        data = json.loads(transcript.raw)

        for word in data['results'][-1]['alternatives'][0]['words']:
            speaker = transcripts.Speaker(int(word['speakerTag']))

            if speaker not in transcript.speakers:
                transcript.speakers.append(speaker)

            new_word = transcripts.Word(
                word['word'],
                startTime=float(word['startTime'][:-1]),
                endTime=float(word['endTime'][:-1]),
                speaker=speaker
            )

            transcript.words.append(new_word)

    def set_vocabulary_file(self, vocabularyFilePath):
        if not self.authenticated:
            self.vocList = None
            raise exceptions.GoogleAuthError()

        with open(vocabularyFilePath) as vocabularyFile:
            self.vocList = vocabularyFile.read().splitlines()

    def _make_language_compatible(self, lang):
        if lang[2] == '-':
            lang = lang[:2].lower() + lang[2:].upper()
        return lang



import requests
import xml.etree.ElementTree as ET

import ftvstt.transcripts as transcripts
import ftvstt.exceptions as exceptions
import ftvstt.transcribers as transcribers


class Vocapia(transcribers.Transcriber):
    rawType = "xml"

    def __init__(self, apiBaseUrl="https://rest1.vocapia.com:8093/voxsigma"):
        super().__init__()

        self.apiBaseUrl = apiBaseUrl
        self.vocabularyFilePath = None

    def __rawType(self):
        return "xml"

    def authenticate(self, user, password):
        self.user = user
        self.password = password
        auth = (self.user, self.password)
        req = requests.post(self.apiBaseUrl, auth=auth)
        self.authenticated = req.status_code == 400
        return self.authenticated

    def deauthenticate(self):
        self.user = None
        self.password = None
        self.authenticated = False

    @exceptions.transcribe_error_handler
    def transcribe(self, inputPath, lang="fr-FR"):
        lang = self._make_language_compatible(lang)
        transcript = transcripts.Transcript()
        transcript.inputPath = inputPath
        transcript.provider = self.__class__

        if not self.authenticated:
            transcript.exception = exceptions.VocapiaAuthError()
            transcript.success = False
            return transcript

        payload = {
            'method': 'vrbs_trans',
            'model': lang
        }
        auth = (self.user, self.password)

        with open(inputPath, 'rb') as inputFile:
            if self.vocabularyFilePath:
                with open(self.vocabularyFilePath, 'rb') as vocabularyFile:
                    files = {
                            'audiofile': inputFile,
                            'vocfile': vocabularyFile
                    }
                    rep = requests.post(
                        self.apiBaseUrl,
                        files=files,
                        auth=auth,
                        data=payload)
            else:
                files = {
                    'audiofile': inputFile
                }
                rep = requests.post(
                    self.apiBaseUrl,
                    files=files,
                    auth=auth,
                    data=payload)

        resultXMLroot = ET.fromstring(rep.text)
        transcript.success = resultXMLroot.tag != 'Error'

        if transcript.success:
            transcript.raw = rep.text
            self.__class__.set_text(transcript)
            self.__class__.set_words(transcript)
        else:
            message = 'Vocapia error '
            message += resultXMLroot.attrib['code']
            message += ' : '
            message += resultXMLroot.text
            transcript.exception = exceptions.VocapiaError(message)

        return transcript

    def set_text(transcript):
        root = ET.fromstring(transcript.raw)
        WordsTag = root.findall('SegmentList/SpeechSegment/Word')
        Words = [word.text for word in WordsTag]
        transcript.text = ""

        escape_chars = ['-', '.', ',', '!', ':', '?']

        for i in range(len(Words)):
            if Words[i][-2] in ['\'', '-']:
                Words[i] = Words[i][:-1]

            if i+1 < len(Words) and Words[i+1][1] in escape_chars:
                Words[i] = Words[i][:-1]

            transcript.text += Words[i][1:]

        return transcript.text

    def set_words(transcript):
        root = ET.fromstring(transcript.raw)
        speakers = root.findall('SpeakerList/Speaker')
        transcript.speakers = []
        for speaker in speakers:
            gender = speaker.attrib['spkid'][0]
            speakerId = int(speaker.attrib['spkid'][2:])
            new_speaker = transcripts.Speaker(speakerId, gender=gender)
            transcript.speakers.append(new_speaker)

        transcript.words = []
        root = ET.fromstring(transcript.raw)
        SpeechSegments = root.findall('SegmentList/SpeechSegment')

        for speechSegment in SpeechSegments:
            gender = speechSegment.attrib['spkid'][0]
            speakerId = int(speechSegment.attrib['spkid'][2:])

            speaker = transcripts.Speaker(speakerId, gender=gender)

            words = list(speechSegment)
            for word in words:
                startTime = float(word.attrib['stime'])

                word = transcripts.Word(
                    word.text.strip(),
                    startTime=startTime,
                    endTime=startTime + float(word.attrib['dur']),
                    speaker=speaker,
                    confidence=float(word.attrib['conf']))

                transcript.words.append(word)
        return transcript.words

    def set_vocabulary_file(self, vocabularyFilePath):
        if not self.authenticated:
            raise exceptions.VocapiaAuthError()

        self.vocabularyFilePath = vocabularyFilePath

    def _make_language_compatible(self, lang):
        if lang.upper() in ["FR-FR", "FRE", "FR"]:
            return "fre"
        elif lang.upper() in ["EN-US", "EN-EN", "ENG", "EN"]:
            return "eng"
        else:
            return lang



import json


class Transcript:
    def __init__(self, raw=None, provider=None):
        self.exception = None
        self.inputPath = None
        self.provider = None
        self.raw = None
        self.rawType = None
        self.speakers = None
        self.success = None
        self.text = None
        self.words = None

        if provider is not None and raw is not None:
            self.provider = provider
            self.raw = raw
            self.rawType = provider.rawType
            provider.set_text(self)
            provider.set_words(self)
            self.success = True

    def dump_raw_result(self, outputPath):
        if not self.success:
            raise self.exception

        with open(outputPath, 'w') as outputFile:
            outputFile.write(self.raw)

    def dump_text_result(self, outputPath):
        if not self.success:
            raise self.exception

        with open(outputPath, 'w') as outputFile:
            outputFile.write(self.text)

    def to_dict(self):
        if not self.success:
            return

        speakers = [
            speaker.__dict__.copy()
            for speaker in self.__dict__['speakers']
        ]
        words = [
            word.__dict__.copy()
            for word in self.__dict__['words']
        ]

        resDict = {
            "provider": self.__dict__['provider'].__name__,
            "speakers": speakers,
            "text": self.__dict__['text'],
            "words": words
        }

        for i in range(len(resDict['words'])):
            speaker = resDict['words'][i]['speaker']
            if speaker is not None:
                resDict['words'][i]['speakerId'] = speaker.id
            else:
                resDict['words'][i]['speakerId'] = None
            del resDict['words'][i]['speaker']

        return resDict

    def dump_normalised_result(self, outputPath):
        if not self.success:
            return

        content = self.to_dict()
        with open(outputPath, mode="w", encoding="utf-8") as file:
            file.write(json.dumps(content, ensure_ascii=False, indent=4))


class Word:
    def __init__(
        self,
        content,
        startTime=None,
        endTime=None,
        speaker=None,
        confidence=None
    ):
        self.content = content
        self.startTime = startTime
        self.endTime = endTime
        self.speaker = speaker
        self.confidence = confidence


class Speaker:
    def __init__(self, id, gender=None):
        self.id = id
        self.gender = gender

    def __eq__(self, other):
        if self.__class__ != other.__class__:
            return False

        return self.__dict__ == other.__dict__




def transcribe_error_handler(transcribe):
    def decorated_transcribe(*args, **kwargs):
        transcript = transcribe(*args, **kwargs)

        if transcript.exception:
            raise transcript.exception

        return transcript
    return decorated_transcribe


class VocapiaError(Exception):
    def __init__(self, message):
        super().__init__(message)


class VocapiaAuthError(Exception):
    def __init__(self):
        super().__init__("No valid Vocapia authentication.")


class BertinError(Exception):
    def __init__(self, message):
        super().__init__(message)


class BertinAuthError(Exception):
    def __init__(self):
        super().__init__("No valid Bertin authentication.")


class GoogleError(Exception):
    def __init__(self, message):
        super().__init__(message)


class GoogleAuthError(Exception):
    def __init__(self):
        super().__init__("No valid Google Cloud authentication.")


class AmazonError(Exception):
    def __init__(self, message):
        super().__init__(message)


class AmazonAuthError(Exception):
    def __init__(self):
        super().__init__("No valid Amazon authentication.")



import requests
from requests.packages.urllib3.exceptions import InsecureRequestWarning
import json

import xml.etree.ElementTree as ET
import os
from time import sleep

import ftvstt.transcripts as transcripts
import ftvstt.exceptions as exceptions
import ftvstt.transcribers as transcribers

requests.packages.urllib3.disable_warnings(InsecureRequestWarning)


class Bertin(transcribers.Transcriber):
    rawType = "xml"

    def __init__(self, apiBaseUrl):
        super().__init__()

        if apiBaseUrl[-1] != '/':
            self.apiBaseUrl = apiBaseUrl + '/'
        else:
            self.apiBaseUrl = apiBaseUrl

        self.__authTokenId = None
        self.__authTokenValue = None
        self.authenticated = False

    def authenticate(self, user, password):
        self.user = user
        self.password = password

        payload = {
            'login': self.user,
            'password': self.password
        }

        logUrl = self.apiBaseUrl + 'auth-tokens'
        rep = requests.post(logUrl, payload, verify=False)
        repDict = json.loads(rep.text)

        self.authenticated = rep.status_code == 201

        if self.authenticated:
            self.__authTokenId = repDict['id']
            self.__authTokenValue = repDict['value']

        return self.authenticated

    def deauthenticate(self):
        self.user = None
        self.password = None

        requests.delete(
            self.apiBaseUrl + "auth-tokens/" + str(self.__authTokenId),
            verify=False
        )

        self.__authTokenId = None
        self.__authTokenValue = None
        self.authenticated = False

    @exceptions.transcribe_error_handler
    def transcribe(self, inputPath, lang="fr-FR"):
        lang = self._make_language_compatible(lang)
        transcript = transcripts.Transcript()
        transcript.inputPath = inputPath
        transcript.provider = self.__class__

        if self.authenticated:
            transcript.exception = exceptions.BertinAuthError()
            transcript.success = False
            return transcript

        inputName = os.path.basename(transcript.inputPath)
        lastDotIndex = inputName.rfind(".")

        # Removing dots inside the name (provoke error in Bertin)
        apiFileName = inputName[:lastDotIndex].replace(".", "_")
        apiFileName += inputName[lastDotIndex:]

        payload = {
            'audioLanguage': lang,
            'apiFileName': apiFileName,
            'deleteResultFilesAfterSuccess': 'false',
            'deleteTmpFilesAfterSuccess': 'true',
            'deleteIfJobFail': 'true',
            'deleteSourceFilesAfterSuccess': 'true',
            'audioType': 'bn',
            'model': '',
            'resultUrl': ''
        }

        headers = {
            'X-Auth-Token': self.__authTokenValue,
            'accept': 'application/json'
        }

        startTranscriptionUrl = self.apiBaseUrl + 'transcriptions/start'
        with open(transcript.inputPath, 'rb') as inputFile:
            repJob = requests.post(
                startTranscriptionUrl,
                files={'apiFile': inputFile},
                headers=headers,
                data=payload,
                verify=False)

        if repJob.status_code != 200:
            message = "Media upload to Bertin failed."
            transcript.exception = exceptions.BertinError(message)
            transcript.success = False
            return transcript

        repJobDict = json.loads(repJob.text)
        jobId = repJobDict["jobId"]
        statusUrl = self.apiBaseUrl + 'transcriptions/status/' + jobId
        repStatus = requests.get(
            statusUrl,
            headers=headers,
            verify=False)

        if repStatus.status_code == 200:
            statusDict = json.loads(repStatus.text)

        while repStatus.status_code != 200 or statusDict['state'] == 1:
            sleep(2)

            repStatus = requests.get(
                statusUrl,
                headers=headers,
                verify=False)

            if repStatus.status_code == 200:
                statusDict = json.loads(repStatus.text)

        sleep(2)
        transcript.success = statusDict['state'] == 3

        if not transcript.success:
            message = "An error has occured "
            message += "during Bertin transciption."
            transcript.exception = exceptions.BertinError(message)
            transcript.success = False
            return transcript

        resultUrl = self.apiBaseUrl
        resultUrl += 'transcriptions/results/'
        resultUrl += jobId
        resultUrl += '/xml'

        repResult = requests.get(
            resultUrl,
            headers=headers,
            verify=False)

        transcript.raw = json.loads(repResult.text)['fileContent']
        self.__class__.set_text(transcript)
        self.__class__.set_words(transcript)

        return transcript

    def set_text(transcript):
        transcript.text = ""
        root = ET.fromstring(transcript.raw)
        Words = root.findall('SegmentList/SpeechSegment/Word')
        for word in Words:
            transcript.text += word.text[1:]
        return transcript.text

    def set_words(transcript):
        root = ET.fromstring(transcript.raw)
        speakers = root.findall('SpeakerList/Speaker')
        transcript.speakers = []
        for speaker in speakers:
            gender = speaker.attrib['spkid'][0]
            speakerId = int(speaker.attrib['spkid'][2:])
            new_speaker = transcripts.Speaker(speakerId, gender=gender)
            transcript.speakers.append(new_speaker)

        transcript.words = []
        root = ET.fromstring(transcript.raw)
        SpeechSegments = root.findall('SegmentList/SpeechSegment')

        for speechSegment in SpeechSegments:
            gender = speechSegment.attrib['spkid'][0]
            speakerId = int(speechSegment.attrib['spkid'][2:])

            speaker = transcripts.Speaker(speakerId, gender=gender)

            words = list(speechSegment)
            for word in words:
                startTime = float(word.attrib['stime'])
                new_word = transcripts.Word(
                    word.text.strip(),
                    startTime=startTime,
                    endTime=startTime + float(word.attrib['dur']),
                    speaker=speaker,
                    confidence=float(word.attrib['conf']))

                transcript.words.append(new_word)
        return transcript.words

    def _make_language_compatible(self, lang):
        if lang.upper() in ["FR-FR", "FRE", "FR"]:
            return "fre"
        elif lang.upper() in ["EN-US", "EN-EN", "ENG", "EN"]:
            return "eng"
        elif len(lang) == 3:
            return lang
        else:
            return "unknown"



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
     name='ftvstt',
     version='2.0.0',
     packages=setuptools.find_packages() ,
     author="France Télévisions innovations et développement",
     author_email="pierre-andre.long@francetv.fr",
     description="Transcription APIs encapsulation",
     long_description=long_description,
   long_description_content_type="text/markdown",
     install_requires=['boto3==1.10.4', 'google-cloud-speech==1.2.0', 'requests==2.22.0'],
     classifiers=[
         "Programming Language :: Python :: 3",
         "License :: OSI Approved :: MIT License",
         "Operating System :: OS Independent",
     ],
 )



import boto3
import requests

import time
import os
import json
import csv

import random

import ftvstt.transcripts as transcripts
import ftvstt.exceptions as exceptions
import ftvstt.transcribers as transcribers


class Amazon(transcribers.Transcriber):
    rawType = "json"

    def __init__(self, bucketName, region_name='eu-west-3'):
        super().__init__()
        self.bucketName = bucketName
        self.region_name = region_name
        self.__vocList = None
        self.__vocName = None

    def authenticate_with_file(self, credentialsFilePath):
        self.credentialsFilePath = credentialsFilePath

        auth = {}
        self.authenticated = True
        try:
            with open(self.credentialsFilePath, mode="r") as csvFile:
                reader = csv.reader(csvFile, delimiter="=")
                for line in reader:
                    auth[line[0]] = line[1]

            self.__s3Client = boto3.client(
                's3',
                aws_access_key_id=auth['AWSAccessKeyId'],
                aws_secret_access_key=auth['AWSSecretKey']
            )

            self.__transcribeClient = boto3.client(
                'transcribe',
                region_name=self.region_name,
                aws_access_key_id=auth['AWSAccessKeyId'],
                aws_secret_access_key=auth['AWSSecretKey']
            )

            self.__s3Client.list_buckets()

        except Exception:
            self.authenticated = False

        return self.authenticated

    def authenticate(self, awsAccessKeyId, awsSecretAccessKey):
        self.id = awsAccessKeyId
        self.password = awsSecretAccessKey

        self.authenticated = True
        try:
            self.__s3Client = boto3.client(
                's3',
                aws_access_key_id=self.id,
                aws_secret_access_key=self.password
            )

            self.__transcribeClient = boto3.client(
                'transcribe',
                region_name=self.region_name,
                aws_access_key_id=self.id,
                aws_secret_access_key=self.password
            )

            self.__s3Client.list_buckets()
        except Exception:
            self.authenticated = False

        return self.authenticated

    def deauthenticate(self):
        self.credentialsFilePath = None
        self.id = None
        self.password = None

        self.__s3Client = None
        self.__transcribeClient = None

    @exceptions.transcribe_error_handler
    # set s3file=True if the input file is already on S3,
    # in that case inputPath should be the url of the file
    def transcribe(self, inputPath, lang="fr-FR", s3file=False):
        lang = self._make_language_compatible(lang)
        transcript = transcripts.Transcript()
        transcript.inputPath = inputPath
        transcript.provider = self.__class__

        if not self.authenticated:
            transcript.exception = exceptions.AmazonAuthError()
            transcript.success = False
        if s3file:
            audioUri = inputPath
            transcript.exception = None
        else:
            s3AudioPath = "ftvstt/"
            s3AudioPath = os.path.basename(transcript.inputPath)
            audioUri, transcript.exception = self.__s3_upload(
                transcript.inputPath,
                s3AudioPath
            )

        if transcript.exception:
            transcript.success = False
            return transcript

        jobName = "ftvstt_job_"
        jobName += str(random.randint(1, 1e6))
        jobName += "_"
        jobName += os.path.basename(transcript.inputPath).replace(".", "_")

        settings = {
            'MaxSpeakerLabels': 10,
            'ShowSpeakerLabels': True
        }
        if not self.__vocList:
            settings['VocabularyName'] = self.__vocName

        try:
            self.__transcribeClient.start_transcription_job(
                TranscriptionJobName=jobName,
                Media={'MediaFileUri': audioUri},
                MediaFormat='wav',
                LanguageCode=lang,
                Settings=settings
            )
        except Exception as e:
            message = "Amazon Transcription failed : \n"
            message += str(e)
            transcript.exception = exceptions.AmazonError(message)
            transcript.success = False
            return transcript

        completed, job = self.__wait_for_transcription_job(jobName)

        if not completed:
            transcript.success = False
            message = "Amazon Transcription failed : \n"
            message += job['TranscriptionJob']['FailureReason']
            transcript.exception = exceptions.AmazonError(message)
            return transcript

        rawResultsUri = (job['TranscriptionJob']
                            ['Transcript']
                            ['TranscriptFileUri'])
        rep = requests.get(rawResultsUri)

        transcript.success = rep.status_code != 404

        if not transcript.success:
            message = "Amazon Transcription failed : \n"
            message += "Impossible to reach transcription result at :"
            message += rawResultsUri
            transcript.exception = exceptions.AmazonError(message)
            return transcript

        transcript.raw = rep.text
        self.__class__.set_text(transcript)
        self.__class__.set_words(transcript)

        time.sleep(2)
        self.__transcribeClient.delete_transcription_job(
            TranscriptionJobName=jobName)

        return transcript

    def set_text(transcript):
        data = json.loads(transcript.raw)
        transcript.text = ""
        for result in data['results']['transcripts']:
            transcript.text += result['transcript']
        return transcript.text

    def set_words(transcript):
        transcript.words = []
        transcript.speakers = []

        data = json.loads(transcript.raw)

        segments = data['results']['speaker_labels']['segments']

        for segment in segments:
            speaker_id = int(segment['speaker_label'][4:])
            speaker = transcripts.Speaker(speaker_id)

            if speaker not in transcript.speakers:
                transcript.speakers.append(speaker)

        for word in data['results']['items']:
            content = word['alternatives'][0]['content']

            if word['type'] == "punctuation":
                if transcript.words == []:
                    startTime = endTime = .0
                else:
                    startTime = transcript.words[-1].endTime
                    endTime = startTime
            else:
                startTime = float(word['start_time'])
                endTime = float(word['end_time'])
            confidence = float(word['alternatives'][0]['confidence'])

            speaker = None

            for segment in segments:
                segmentStartTime = float(segment['start_time'])
                segmentEndTime = float(segment['end_time'])
                if startTime >= segmentStartTime and endTime <= segmentEndTime:
                    speakerId = int(segment['speaker_label'][4:])
                    speaker = transcripts.Speaker(speakerId)
                    break

            newWord = transcripts.Word(
                content,
                startTime=startTime,
                endTime=endTime,
                confidence=confidence,
                speaker=speaker
            )

            transcript.words.append(newWord)

    def __wait_for_transcription_job(self, jobName):
        while True:
            job = self.__transcribeClient.get_transcription_job(
                TranscriptionJobName=jobName)

            status = job['TranscriptionJob']['TranscriptionJobStatus']

            if status in ['COMPLETED', 'FAILED']:
                break
            time.sleep(1)
        return status == 'COMPLETED', job

    def __s3_upload(self, inputPath, s3Path):
        exception = None
        fileUri = None
        try:
            self.__s3Client.upload_file(inputPath, self.bucketName, s3Path)

            pattern = "https://%s.s3.%s.amazonaws.com/%s"
            fileUri = pattern % self.bucketName, self.region_name, s3Path

        except Exception as e:
            message = "Media upload to Amazon failed : \n" + str(e)
            exception = exceptions.AmazonError(message)
        return fileUri, exception

    def set_vocabulary_file(self, vocabularyFilePath, lang="fr-FR"):
        exception = None
        if self.authenticated:
            with open(vocabularyFilePath) as vocabularyFile:
                self.__vocList = vocabularyFile.read().splitlines()

            filename = "ftvstt_voc_"
            filename = str(random.randint(1, 1e6))
            filename = "_"
            filename += os.path.basename(vocabularyFilePath).replace(".", "_")
            self.__vocName = filename
            exception = self.__create_vocabulary_file(lang)
        else:
            exception = exceptions.AmazonAuthError()

        if exception:
            self.__vocList = None
            raise exception

    def __create_vocabulary_file(self, lang):
        exception = None
        self.__transcribeClient.create_vocabulary(
            VocabularyName=self.__vocName,
            LanguageCode=lang,
            Phrases=self.__vocList
        )

        while True:
            status = self.__transcribeClient.get_vocabulary(
                VocabularyName=self.__vocName)

            if status['VocabularyState'] in ['READY', 'FAILED']:
                break
            time.sleep(1)

        if status['VocabularyState'] == 'FAILED':
            message = "Couldn't upload vocabulary file : \n"
            message += status['FailureReason']
            exception = exceptions.AmazonError(message)
        return exception

    def _make_language_compatible(self, lang):
        if lang[2] == '-':
            lang = lang[:2].lower() + lang[2:].upper()
        return lang
